(module sudoku-tp mzscheme
  (require (lib "error.ss" "htdp")
           "image.ss"
           (lib "mred.ss" "mred")
           (lib "etc.ss")
           (lib "class.ss")
           (prefix beg: (lib "htdp-beginner.ss" "lang")))
  
  
  (provide (all-from "image.ss")
           with-image
           build-vector
           vector->list
           vector-length
           vector-ref)
  
  (define-syntax (with-image stx)
    (syntax-case stx ()
      [(_ a1 a2)
       (syntax (with-image/proc a1 (lambda () a2)))]))
  
  (define (with-image/proc p continue)
    (check-arg 'with-image (beg:image? p) "image" "first" p)
    (set! picts (append picts (list p)))
    (build-frame/update p)
    (continue))
  
  (define the-frame #f)
  (define txt (new text%))

  (define (prev)
    (set! index (modulo (- index 1) (length picts)))
    (update))
  (define (next) 
    (set! index (modulo (+ index 1) (length picts)))
    (update))
  
  (define (update) (update-frame (list-ref picts index)))
  
  (define picts '())
  (define index 0)
  
  (define show-eventspace (make-eventspace))
  
  (define (build-frame/update p)
    (cond
      [the-frame
       (queue-pict p)]
      [else
       (parameterize ([current-eventspace show-eventspace])
         (set! the-frame (new frame% (label "DrScheme")))
         (queue-callback
          (lambda () 
            (let ([hp (new horizontal-panel% 
                           (parent the-frame)
                           (alignment '(center center))
                           (stretchable-height #f))])
              (new button% (parent hp) (callback (lambda (x y) (prev))) (label " << "))
              (new button% (parent hp) (callback (lambda (x y) (next))) (label " >> ")))
            
            (send 
             (new editor-canvas%
                  (parent the-frame) 
                  (editor txt)
                  (style '(no-hscroll no-vscroll))
                  ;; this 20 stuff is a hack, for now
                  (min-width (+ (image-width p) 20))
                  (min-height (+ (image-height p) 20)))
             focus)
            (update-frame p)
            (send the-frame show #t)
            (send txt hide-caret #t))))]))

  (define callback-pict #f)
  (define sema (make-semaphore 1))
  
  (define (queue-pict pict)
    (semaphore-wait sema)
    (cond
      [callback-pict
       (set! callback-pict pict)
       (semaphore-post sema)]
      [else
       (set! callback-pict pict)
       (semaphore-post sema)
       (parameterize ([current-eventspace show-eventspace])
         (queue-callback
          (lambda ()
            ; (sleep 1) ;; doesn't really seem to help (if it did, it should be in a separate thread ...
            (let ([pict #f])
              (semaphore-wait sema)
              (set! pict callback-pict)
              (set! callback-pict #f)
              (semaphore-post sema)
              (update-frame pict)))
          #f))]))
  
  (define (update-frame pict)
    (send txt begin-edit-sequence)
    (send txt lock #f)
    (send txt delete 0 (send txt last-position) #f)
    (send txt insert (send pict copy) 0 0 #f)
    (send txt lock #t)
    (send txt end-edit-sequence)))
